@@ -13,7 +13,7 @@
 #' @export
 #' @import data.table
 #' @importFrom stars read_stars st_dimensions
-#' @importFrom sf st_bbox st_crs
+#' @importFrom sf st_bbox st_crs st_as_text gdal_utils
 #' @importFrom methods is
 #' @examples
 #' # Define product names
@@ -80,8 +80,9 @@ raster_metadata <- function(raster_paths, meta = "all", format = "data.table") {
     }
     if (meta_gdalinfo) {
       metadata_raw <- suppressWarnings(suppressMessages(try(
-        sf::gdal_utils("info", raster_path, quiet = TRUE) %>%
-          strsplit("\n") %>% unlist() %>% trimws(),
+        trimws(unlist(
+          strsplit(gdal_utils("info", raster_path, quiet = TRUE), "\n")
+        )),
         silent = TRUE
       )))
     }
@@ -131,17 +132,21 @@ raster_metadata <- function(raster_paths, meta = "all", format = "data.table") {
         out_list[[i]][["proj"]] <- ref_proj
       }
       if ("unit" %in% meta) {
-        out_list[[i]][["unit"]] <- as.character(projpar(ref_proj$proj4string, "unit"))
+        out_list[[i]][["unit"]] <- as.character(projpar(ref_proj, "unit"))
       }
       
       if ("outformat" %in% meta) {
-        out_list[[i]][["outformat"]] <- metadata_raw[grepl("Driver:", metadata_raw)] %>%
-          gsub("Driver: ?([A-Za-z0-9_]+)/.*$", "\\1", .)
+        out_list[[i]][["outformat"]] <- gsub(
+          "Driver: ?([A-Za-z0-9_]+)/.*$", "\\1",
+          metadata_raw[grepl("Driver:", metadata_raw)]
+        )
       }
       
       if ("type" %in% meta) {
-        out_list[[i]][["type"]] <- metadata_raw[grepl("Band [0-9]+.+Type ?=", metadata_raw)][1] %>%
-          gsub("Band [0-9]+.+Type ?= ?([A-Za-z0-9]+),.*$", "\\1", .)
+        out_list[[i]][["type"]] <- gsub(
+          "Band [0-9]+.+Type ?= ?([A-Za-z0-9]+),.*$", "\\1",
+          metadata_raw[grepl("Band [0-9]+.+Type ?=", metadata_raw)][1]
+        )
       }
       
       # if (format %in% c("data.frame", "data.table")) {
@@ -202,7 +207,15 @@ raster_metadata <- function(raster_paths, meta = "all", format = "data.table") {
           sel_dt$xmax <- l$bbox["xmax"]
           sel_dt$ymax <- l$bbox["ymax"]
         }
-        if ("proj" %in% meta) {sel_dt$proj <- l$proj$proj4string}
+        if ("proj" %in% meta) {
+          sel_dt$proj <- if (!is.na(l$proj$epsg)) {
+            paste0("EPSG:",l$proj$epsg)
+          } else if (!is.na(l$proj)) {
+            st_as_text_2(l$proj)
+          } else {
+            NA
+          }
+        }
         if ("unit" %in% meta) {sel_dt$unit <- l$unit}
         if ("outformat" %in% meta) {sel_dt$outformat <- l$outformat}
         if ("type" %in% meta) {sel_dt$type <- l$type}
