@@ -41,7 +41,7 @@ testthat::test_that(
       exp_meta_r[,c("xmin", "xmax", "ymin", "ymax")], 
       data.table("xmin" = 578590, "xmax" = 597700, "ymin" = 5086740, "ymax" = 5101530) 
     )
-    testthat::expect_equal(sf::st_crs(exp_meta_r$proj)$epsg, 32632)
+    testthat::expect_equal(st_crs2(exp_meta_r$proj)$epsg, 32632)
     testthat::expect_equal(exp_meta_r$type, "UInt16")
     testthat::expect_equal(exp_meta_r$outformat, "GTiff") # default value
     
@@ -53,9 +53,10 @@ testthat::test_that(
     testthat::expect_equal(exp_meta_s$extent_name, "Scalve")
     
     # test on raster values
-    r <- raster::brick(exp_outpath_2)
-    testthat::expect_equal(raster::cellStats(r[[3]], "mean"), 3800.772, tolerance = 1e-3)
-    testthat::expect_equal(raster::cellStats(r[[3]], "countNA"), 1417518, tolerance = 1e-03)
+    exp_stars <- stars::read_stars(exp_outpath_2)
+    testthat::expect_equal(mean(exp_stars[[1]][,,3], na.rm=TRUE), 3800.772, tolerance = 1e-03)
+    testthat::expect_equal(sum(is.na(exp_stars[[1]][,,3])), 1417518, tolerance = 1e-03)
+    rm(exp_stars)
     
     # test thumbnails
     exp_outpath_t_2 <- file.path(
@@ -83,8 +84,8 @@ testthat::test_that(
       data.table(exp_meta_r[,c("xmin", "xmax", "ymin", "ymax")])
     )
     testthat::expect_equal(
-      sf::st_crs(exp_meta_r_t$proj)$epsg, 
-      sf::st_crs(exp_meta_r$proj)$epsg
+      st_crs2(exp_meta_r_t$proj)$epsg, 
+      st_crs2(exp_meta_r$proj)$epsg
     )
     testthat::expect_equal(exp_meta_r_t$type, "Byte")
     testthat::expect_equal(exp_meta_r_t$outformat, "JPEG")
@@ -119,7 +120,10 @@ testthat::test_that(
         path_subdirs = FALSE,
         overwrite = TRUE
       ),
-      regexp = "[Bb]oth native and custom resolution were provided" # FIXME 
+      regexp = gsub(
+        " ", "[ \n]",
+        "[Bb]oth native and custom resolution were provided"
+      )
     )
     expect_true(all(file.exists(c(
       exp_outpath_3,
@@ -141,7 +145,7 @@ testthat::test_that(
       data.frame("xmin" = 113909, "xmax" = 133284, "ymin" = 5097856, "ymax" = 5112431),
       tolerance = 1e-3
     )
-    testthat::expect_equal(sf::st_crs(exp_meta_r$proj)$epsg, 32633)
+    testthat::expect_equal(st_crs2(exp_meta_r$proj)$epsg, 32633)
     testthat::expect_equal(exp_meta_r$type, "UInt16")
     testthat::expect_equal(exp_meta_r$outformat, "ENVI")
     
@@ -153,9 +157,10 @@ testthat::test_that(
     testthat::expect_equal(exp_meta_s$extent_name, "Scalve")
     
     # test on raster values
-    r <- raster::brick(exp_outpath_3)
-    testthat::expect_equal(raster::cellStats(r[[3]], "mean"), 2316, tolerance = 1e-03)
-    testthat::expect_equal(raster::cellStats(r[[3]], "countNA"), 0)
+    exp_stars <- stars::read_stars(exp_outpath_3)
+    testthat::expect_equal(mean(exp_stars[[1]][,,3], na.rm=TRUE), 2316, tolerance = 1e-03)
+    testthat::expect_equal(sum(is.na(exp_stars[[1]][,,3])), 0)
+    rm(exp_stars)
     
     # test thumbnails
     exp_outpath_t_3 <- file.path(
@@ -181,8 +186,8 @@ testthat::test_that(
       data.frame(exp_meta_r[,c("xmin", "xmax", "ymin", "ymax")])
     )
     testthat::expect_equal(
-      sf::st_crs(exp_meta_r_t$proj)$epsg, 
-      sf::st_crs(exp_meta_r$proj)$epsg
+      st_crs2(exp_meta_r_t$proj)$epsg, 
+      st_crs2(exp_meta_r$proj)$epsg
     )
     testthat::expect_equal(exp_meta_r_t$type, "Byte")
     testthat::expect_equal(exp_meta_r_t$outformat, "JPEG")
@@ -244,9 +249,10 @@ testthat::test_that(
     testthat::expect_equal(exp_meta_s$extent_name, "Scalve")
     
     # test on raster values
-    r <- raster::raster(exp_outpath_4)
-    testthat::expect_equal(raster::cellStats(r, "max"), 11)
-    testthat::expect_equal(raster::cellStats(r, "countNA"), 0)
+    exp_stars <- stars::read_stars(exp_outpath_4)
+    testthat::expect_equal(max(exp_stars[[1]], na.rm=TRUE), 11, tolerance = 1e-03)
+    testthat::expect_equal(sum(is.na(exp_stars[[1]])), 0, tolerance = 1e-03)
+    rm(exp_stars)
     
     # test thumbnails
     exp_outpath_t_4 <- file.path(
@@ -263,8 +269,8 @@ testthat::test_that(
     testthat::expect_equal(exp_meta_r_t$nbands, 3)
     testthat::expect_equal(exp_meta_r_t$bbox, exp_meta_r$bbox)
     testthat::expect_equal(
-      sf::st_crs(exp_meta_r_t$proj)$epsg, 
-      sf::st_crs(exp_meta_r$proj)$epsg
+      st_crs2(exp_meta_r_t$proj)$epsg, 
+      st_crs2(exp_meta_r$proj)$epsg
     )
     testthat::expect_equal(exp_meta_r_t$type, "Byte")
     testthat::expect_equal(exp_meta_r_t$outformat, "PNG")
@@ -310,20 +316,21 @@ testthat::test_that(
       exp_meta_r[,c("xmin", "xmax", "ymin", "ymax")], 
       data.frame("xmin" = 580560, "xmax" = 580800, "ymin" = 5101700, "ymax" = 5102120)
     )
-    testthat::expect_equal(sf::st_crs(exp_meta_r$proj)$epsg, 32632)
+    testthat::expect_equal(st_crs2(exp_meta_r$proj)$epsg, 32632)
     testthat::expect_equal(exp_meta_r$type, "Byte")
     testthat::expect_equal(exp_meta_r$outformat, "GTiff")
     
     # expect error on sen2r metadata with unstandard name
     exp_meta_s <- testthat::expect_error(
       sen2r_getElements(exp_outpath_4b),
-      regexp = "not recognised"
+      regexp = "not[ \n]recognised"
     )
     
     # test on raster values
-    r <- raster::brick(exp_outpath_4b)
-    testthat::expect_equal(raster::cellStats(r[[3]], "mean"), 105.0556, tolerance = 1e-03)
-    testthat::expect_equal(raster::cellStats(r[[3]], "countNA"), 0, tolerance = 1e-03)
+    exp_stars <- stars::read_stars(exp_outpath_4b)
+    testthat::expect_equal(mean(exp_stars[[1]][,,3], na.rm=TRUE), 105.0556, tolerance = 1e-03)
+    testthat::expect_equal(sum(is.na(exp_stars[[1]][,,3])), 0, tolerance = 1e-03)
+    rm(exp_stars)
     
   }
 )
@@ -359,14 +366,15 @@ testthat::test_that(
     testthat::expect_equal(exp_meta_r$outformat, "GTiff")
     
     # test on raster values
-    r <- raster::brick(test1)
-    testthat::expect_equal(raster::cellStats(r[[3]], "mean"), 100.6298, tolerance = 1e-03)
-    testthat::expect_equal(raster::cellStats(r[[3]], "countNA"), 0)
+    exp_stars <- stars::read_stars(test1)
+    testthat::expect_equal(mean(exp_stars[[1]][,,3], na.rm=TRUE), 100.6298, tolerance = 1e-03)
+    testthat::expect_equal(sum(is.na(exp_stars[[1]][,,3])), 0)
+    rm(exp_stars)
     
     # tests on sen2r metadata
     exp_meta_s <- testthat::expect_error(
       sen2r_getElements(test1),
-      regexp = "not recognised"
+      regexp = "not[ \n]recognised"
     )
     
   }
@@ -392,9 +400,10 @@ testthat::test_that(
     testthat::expect_equal(exp_meta_r$outformat, "GTiff")
     
     # test on raster values
-    r <- raster::brick(test2)
-    testthat::expect_equal(raster::cellStats(r[[3]], "mean"), 122.4337, tolerance = 1e-03)
-    testthat::expect_equal(raster::cellStats(r[[3]], "countNA"), 125, tolerance = 1e-03)
+    exp_stars <- stars::read_stars(test2)
+    testthat::expect_equal(mean(exp_stars[[1]][,,3], na.rm=TRUE), 122.4337, tolerance = 1e-03)
+    testthat::expect_equal(sum(is.na(exp_stars[[1]][,,3])), 125, tolerance = 1e-03)
+    rm(exp_stars)
     
   }
 )
@@ -419,9 +428,10 @@ testthat::test_that(
     testthat::expect_equal(exp_meta_r$outformat, "GTiff")
     
     # test on raster values
-    r <- raster::brick(test3)
-    testthat::expect_equal(raster::cellStats(r[[3]], "mean"), 103.4643, tolerance = 1e-03)
-    testthat::expect_equal(raster::cellStats(r[[3]], "countNA"), 0)
+    exp_stars <- stars::read_stars(test3)
+    testthat::expect_equal(mean(exp_stars[[1]][,,3], na.rm=TRUE), 103.4643, tolerance = 1e-03)
+    testthat::expect_equal(sum(is.na(exp_stars[[1]][,,3])), 0)
+    rm(exp_stars)
     
   }
 )
@@ -430,7 +440,7 @@ testthat::test_that(
   "Reproject all the input file", {
     
     test4 <- tempfile(fileext = "_test4.tif")
-    gdal_warp(ex_sel, test4, t_srs = sf::st_crs("+init=epsg:32631")$proj4string)
+    gdal_warp(ex_sel, test4, t_srs = 32631)
     
     # test on raster metadata
     exp_meta_r <- raster_metadata(test4, format = "list")[[1]]
@@ -447,9 +457,58 @@ testthat::test_that(
     testthat::expect_equal(exp_meta_r$outformat, "GTiff")
     
     # test on raster values
-    r <- raster::brick(test4)
-    testthat::expect_equal(raster::cellStats(r[[3]], "mean"), 104.3775, tolerance = 1e-03)
-    testthat::expect_equal(raster::cellStats(r[[3]], "countNA"), 176, tolerance = 1e-3)
+    exp_stars <- stars::read_stars(test4)
+    testthat::expect_equal(mean(exp_stars[[1]][,,3], na.rm=TRUE), 104.3775, tolerance = 1e-03)
+    testthat::expect_equal(sum(is.na(exp_stars[[1]][,,3])), 176, tolerance = 1e-3)
+    rm(exp_stars)
+    
+  }
+)
+
+testthat::test_that(
+  "Reproject in a projection without EPSG", {
+    # this test is intended to test gdal_warp() passing a WKT to gdalwarp
+    # instead then the EPSG code
+    
+    modis_wkt <- paste0(
+      'PROJCS["Sinusoidal",GEOGCS["GCS_Undefined",DATUM["Undefined",SPHEROID[',
+      '"User_Defined_Spheroid",6371007.181,0.0]],PRIMEM["Greenwich",0.0],UNIT',
+      '["Degree",0.0174532925199433]],PROJECTION["Sinusoidal"],PARAMETER["Fal',
+      'se_Easting",0.0],PARAMETER["False_Northing",0.0],PARAMETER["Central_Me',
+      'ridian",0.0],UNIT["Meter",1.0]]'
+    )
+    test4b <- tempfile(fileext = "_test4b.tif")
+    test4_out <- tryCatch(
+      gdal_warp(ex_sel, test4b, t_srs = modis_wkt),
+      warning = function(w) {
+        suppressWarnings(gdal_warp(ex_sel, test4b, t_srs = modis_wkt))
+        w$message
+      }
+    )
+    testthat::expect_true(any(
+      test4_out == 0,
+      grepl("Discarded datum unknown in CRS definition", test4_out)
+    ))
+    
+    # test on raster metadata
+    exp_meta_r <- raster_metadata(test4b, format = "list")[[1]]
+    testthat::expect_equal(exp_meta_r$size, c("x"=27, "y"=40))
+    testthat::expect_equal(exp_meta_r$res, c("x"=10.64168, "y"=10.58520), tolerance = 1e-3)
+    testthat::expect_equal(exp_meta_r$nbands, 3)
+    testthat::expect_equal(
+      as.numeric(exp_meta_r$bbox), 
+      c(774691.9, 5122100.0,  774979.2, 5122523.4),
+      tolerance = 1e-3
+    )
+    testthat::expect_equal(exp_meta_r$proj$epsg, as.integer(NA))
+    testthat::expect_equal(exp_meta_r$type, "Byte")
+    testthat::expect_equal(exp_meta_r$outformat, "GTiff")
+    
+    # test on raster values
+    exp_stars <- stars::read_stars(test4b)
+    testthat::expect_equal(mean(exp_stars[[1]][,,3], na.rm=TRUE), 104.9611, tolerance = 1e-03)
+    testthat::expect_equal(sum(is.na(exp_stars[[1]][,,3])), 180, tolerance = 1e-3)
+    rm(exp_stars)
     
   }
 )
@@ -461,7 +520,7 @@ testthat::test_that(
     test5 <- tempfile(fileext = "_test5.tif")
     gdal_warp(
       ex_sel, test5, 
-      t_srs = sf::st_crs("+init=epsg:32631")$proj4string, 
+      t_srs = "EPSG:32631", 
       mask = stars::read_stars(test1)
     )
     
@@ -480,9 +539,10 @@ testthat::test_that(
     testthat::expect_equal(exp_meta_r$outformat, "GTiff")
     
     # test on raster values
-    r <- raster::brick(test5)
-    testthat::expect_equal(raster::cellStats(r[[3]], "mean"), 100.7103, tolerance = 1e-03)
-    testthat::expect_equal(raster::cellStats(r[[3]], "countNA"), 56, tolerance = 1e-3)
+    exp_stars <- stars::read_stars(test5)
+    testthat::expect_equal(mean(exp_stars[[1]][,,3], na.rm=TRUE), 100.7103, tolerance = 1e-03)
+    testthat::expect_equal(sum(is.na(exp_stars[[1]][,,3])), 56, tolerance = 1e-03)
+    rm(exp_stars)
     
   }
 )
@@ -493,7 +553,7 @@ testthat::test_that(
     
     gdal_warp(
       ex_sel, test6, 
-      t_srs = sf::st_crs("+init=epsg:32631")$proj4string, 
+      t_srs = 32631, 
       mask = crop_poly
     )
     
@@ -512,9 +572,10 @@ testthat::test_that(
     testthat::expect_equal(exp_meta_r$outformat, "GTiff")
     
     # test on raster values
-    r <- raster::brick(test6)
-    testthat::expect_equal(raster::cellStats(r[[3]], "mean"), 123.5974, tolerance = 1e-03)
-    testthat::expect_equal(raster::cellStats(r[[3]], "countNA"), 73, tolerance = 1e-3)
+    exp_stars <- stars::read_stars(test6)
+    testthat::expect_equal(mean(exp_stars[[1]][,,3], na.rm=TRUE), 123.5974, tolerance = 1e-03)
+    testthat::expect_equal(sum(is.na(exp_stars[[1]][,,3])), 73, tolerance = 1e-03)
+    rm(exp_stars)
     
   }
 )
@@ -543,9 +604,10 @@ testthat::test_that(
     testthat::expect_equal(exp_meta_r$outformat, "GTiff")
     
     # test on raster values
-    r <- raster::brick(test7)
-    testthat::expect_equal(raster::cellStats(r[[3]], "mean"), 99.14, tolerance = 1e-03)
-    testthat::expect_equal(raster::cellStats(r[[3]], "countNA"), 0)
+    exp_stars <- stars::read_stars(test7)
+    testthat::expect_equal(mean(exp_stars[[1]][,,3], na.rm=TRUE), 99.14, tolerance = 1e-03)
+    testthat::expect_equal(sum(is.na(exp_stars[[1]][,,3])), 0)
+    rm(exp_stars)
     
   }
 )
@@ -571,9 +633,10 @@ testthat::test_that(
     testthat::expect_equal(exp_meta_r$outformat, "GTiff")
     
     # test on raster values
-    r <- raster::brick(test8)
-    testthat::expect_equal(raster::cellStats(r[[3]], "mean"), 97.9814, tolerance = 1e-03)
-    testthat::expect_equal(raster::cellStats(r[[3]], "countNA"), 19, tolerance = 1e-3)
+    exp_stars <- stars::read_stars(test8)
+    testthat::expect_equal(mean(exp_stars[[1]][,,3], na.rm=TRUE), 97.9814, tolerance = 1e-03)
+    testthat::expect_equal(sum(is.na(exp_stars[[1]][,,3])), 19, tolerance = 1e-03)
+    rm(exp_stars)
     
   }
 )
@@ -599,9 +662,10 @@ testthat::test_that(
     testthat::expect_equal(exp_meta_r$outformat, "GTiff")
     
     # test on raster values
-    r <- raster::brick(test9)
-    testthat::expect_equal(raster::cellStats(r[[3]], "mean"), 114.433, tolerance = 1e-03)
-    testthat::expect_equal(raster::cellStats(r[[3]], "countNA"), 53, tolerance = 1e-3)
+    exp_stars <- stars::read_stars(test9)
+    testthat::expect_equal(mean(exp_stars[[1]][,,3], na.rm=TRUE), 114.433, tolerance = 1e-03)
+    testthat::expect_equal(sum(is.na(exp_stars[[1]][,,3])), 53, tolerance = 1e-03)
+    rm(exp_stars)
     
   }
 )
