@@ -49,7 +49,7 @@ testthat::test_that(
       exp_meta_r[1,c("xmin", "xmax", "ymin", "ymax")], 
       data.table("xmin" = 578590, "xmax" = 597700, "ymin" = 5086740, "ymax" = 5101530) 
     )
-    testthat::expect_equal(sf::st_crs(exp_meta_r$proj[2])$epsg, 32632)
+    testthat::expect_equal(st_crs2(exp_meta_r$proj[2])$epsg, 32632)
     testthat::expect_equal(exp_meta_r$type, c("UInt16",rep("Byte",3)))
     testthat::expect_equal(exp_meta_r$outformat, rep("GTiff",4)) # default value
     
@@ -92,8 +92,8 @@ testthat::test_that(
       exp_meta_r[,c("xmin", "xmax", "ymin", "ymax")]
     )
     testthat::expect_equal(
-      lapply(exp_meta_r_t$proj, function(x){sf::st_crs(x)$epsg}),
-      lapply(exp_meta_r$proj, function(x){sf::st_crs(x)$epsg})
+      lapply(exp_meta_r_t$proj, function(x){st_crs2(x)$epsg}),
+      lapply(exp_meta_r$proj, function(x){st_crs2(x)$epsg})
     )
     testthat::expect_equal(exp_meta_r_t$type, rep("Byte",4))
     testthat::expect_equal(exp_meta_r_t$outformat, rep("JPEG",4))
@@ -121,7 +121,7 @@ testthat::test_that(
       rgb_bands = list(c(11,8,4),c(9,5,4)),
       scaleRange = list(c(0,7500), matrix(c(rep(0,3),8500,6000,4000),ncol=2)),
       outdir = outdir_17,
-      compress = 50,
+      compress = 90,
       parallel = FALSE
     )
     testthat::expect_true(all(file.exists(exp_outpath_17)))
@@ -141,7 +141,7 @@ testthat::test_that(
       exp_meta_r[1,c("xmin", "xmax", "ymin", "ymax")], 
       data.table("xmin" = 580560, "xmax" = 580800, "ymin" = 5101700, "ymax" = 5102120) 
     )
-    testthat::expect_equal(sf::st_crs(exp_meta_r$proj[2])$epsg, 32632)
+    testthat::expect_equal(st_crs2(exp_meta_r$proj[2])$epsg, 32632)
     testthat::expect_equal(exp_meta_r$type, rep("Byte",2))
     testthat::expect_equal(exp_meta_r$outformat, rep("GTiff",2)) # default value
     
